#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "util.h"

void print_help(char **argv);
void setup(klist *ctx, int argc, char **argv);

int main(int argc, char **argv) {
    klist *ctx = klist_init("test.db");

    if (argc == 1) {
        print_help(argv);
        ctx->error = 1;
    }

    if (!ctx->error) {
        setup(ctx, argc, argv);
        switch (ctx->cmd) {
            case USER:
                switch (((klist_list_context *)ctx->cmd_ctx)->cmd) {
                case USER_GET:
                case USER_DELETE:
                default: ;
                }
            case LIST:
                switch (((klist_user_context *)ctx->cmd_ctx)->cmd) {
                case LIST_ADD:
                case LIST_EDIT:
                case LIST_GET:
                case LIST_DELETE:
                default: ;
                }
            case TASK:
                switch (((klist_task_context *)ctx->cmd_ctx)->cmd) {
                case TASK_ADD:
                case TASK_EDIT:
                case TASK_GET:
                case TASK_DELETE:
                default: ;
                }
            default: ;
        }
    }

    const int error = ctx->error;
    klist_deinit(ctx);
    return error;
}

void print_help(char **argv) {
    fprintf(stderr, "Usage: %s <action>\n", argv[0]);
}

void setup(klist *ctx, int argc, char **argv) {
    if (strcmp(argv[1], "user") == 0)
        ctx->cmd = USER;
    else if (strcmp(argv[1], "list") == 0)
        ctx->cmd = LIST;
    else if (strcmp(argv[1], "task") == 0)
        ctx->cmd = TASK;
    else print_help(argv);

    int opt;
    switch (ctx->cmd) {
        case USER:
            optind = 2;
            while ((opt = getopt(argc, argv, "dg")) != -1)
                switch (opt) {
                    case 'd':
                        fprintf(stderr, "Deleting user %u\n", getuid());
                        break;
                    case 'g':
                        fprintf(stderr, "Printing user lists\n");
                        break;
                    default:
                        print_help(argv);
                }

            break;
        case LIST:
            if (argc < 3) {
                fprintf(stderr, "Prints all lists.\n");
                break;
            }
            klist_list_context *list_ctx = malloc(sizeof(struct klist_list_context));

            optind = 3;
            while ((opt = getopt(argc, argv, "aeds:")) != -1)
                switch (opt) {
                    case 'a':
                        list_ctx->cmd = LIST_ADD;
                        break;
                    case 'e':
                        list_ctx->cmd = LIST_EDIT;
                        break;
                    case 'd':
                        list_ctx->cmd = LIST_DELETE;
                        break;
                    default:
                        print_help(argv);
                        fprintf(stderr, ""
                                "list options:"
                                "-a\tadd a list"
                                "-e\tedit a list"
                                "-d\tdelete a list"
                                "Not providing a parameter prints all lists.");
            }
            ctx->cmd_ctx = list_ctx;
            break;
        case TASK:
            if (argc < 3) {
                fprintf(stderr, "Prints all lists.\n");
                break;
            }
            klist_list_context *task_ctx = malloc(sizeof(struct klist_task_context));

            optind = 3;
            while ((opt = getopt(argc, argv, "aeds:")) != -1)
                switch (opt) {
                    case 'a':
                        task_ctx->cmd = TASK_ADD;
                        break;
                    case 'e':
                        task_ctx->cmd = TASK_EDIT;
                        break;
                    case 'd':
                        task_ctx->cmd = TASK_DELETE;
                        break;
                    default:
                        print_help(argv);
                        fprintf(stderr, ""
                                "task options:"
                                "-a\tadd a task"
                                "-e\tedit a task"
                                "-d\tdelete a task"
                                "Not providing a parameter prints all tasks of a list.");
                }

            ctx->cmd_ctx = task_ctx;
            break;
        default:
            fprintf(stderr, "How did we land here?! Pls report argv[1] = %s\n", argv[1]);
            print_help(argv);
    }
}